import React from 'react';
import classnames from 'classnames';
import { Heading, Body } from '@highlight-ui/typography';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import { Icon } from '@highlight-ui/icon';
import { Tooltip } from '@highlight-ui/tooltip';
import { Input, TextArea } from '@highlight-ui/input';
import { useId } from '@highlight-ui/utils-hooks';
import { Select } from '@highlight-ui/select';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
function __rest(s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
var styles$5 = {
  "horizontalFormField": "HorizontalFormField-module__myzUZuZQ__v7-3-4",
  "head": "HorizontalFormField-module__gsUtslNo__v7-3-4",
  "field": "HorizontalFormField-module__XlUwbuK7__v7-3-4",
  "validations": "HorizontalFormField-module__5W6Ekyt9__v7-3-4",
  "tooltipContent": "HorizontalFormField-module__Bcugbben__v7-3-4"
};
var styles$4 = {
  "labelContainer": "Label-module__NGRSZuBR__v7-3-4",
  "labelTooltipIcon": "Label-module__cQ-uGnUZ__v7-3-4",
  "hintText": "Label-module__YTw7wAe1__v7-3-4"
};
var Label = function (_a) {
  var label = _a.label,
    tooltipContent = _a.tooltipContent,
    tooltipMouseOutDelay = _a.tooltipMouseOutDelay,
    htmlFor = _a.htmlFor,
    hintText = _a.hintText;
  return /*#__PURE__*/React.createElement("div", {
    className: styles$4.labelContainer
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: htmlFor
  }, /*#__PURE__*/React.createElement(Heading, {
    variant: "small",
    as: "span",
    color: "text-default"
  }, label)), tooltipContent && /*#__PURE__*/React.createElement(Tooltip, {
    content: /*#__PURE__*/React.createElement(Body, null, tooltipContent),
    mouseOutDelay: tooltipMouseOutDelay,
    metadata: {
      testId: 'form-field-label-tooltip-icon'
    }
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Icon, {
    className: styles$4.labelTooltipIcon,
    name: "info-circle",
    color: "icon-default",
    type: "solid",
    size: 14
  }))), hintText && /*#__PURE__*/React.createElement("div", {
    className: styles$4.hintText
  }, /*#__PURE__*/React.createElement(Body, {
    variant: "small",
    color: "text-hint-text"
  }, "(", hintText, ")")));
};
var styles$3 = {
  "validations": "Validations-module__9UkiOjPl__v7-3-4"
};
var styles$2 = {
  "validationStatusIcon": "ValidationMessage-module__IK9-Ul8V__v7-3-4"
};
var validationStatusesIconProps = {
  validating: {
    name: 'spinner-third',
    color: 'icon-highlight',
    spin: true
  },
  valid: {
    name: 'check-circle',
    color: 'icon-success'
  },
  invalid: {
    name: 'exclamation-circle',
    color: 'icon-critical'
  },
  warning: {
    name: 'exclamation-triangle',
    color: 'icon-warning'
  },
  info: {
    name: 'info-circle',
    color: 'icon-highlight'
  }
};
var ValidationMessage = function (_a) {
  var className = _a.className,
    message = _a.message,
    status = _a.status,
    metadata = _a.metadata;
  var validationStatusIconProps = validationStatusesIconProps[status];
  return /*#__PURE__*/React.createElement("div", __assign({
    className: className
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement(Icon, __assign({
    className: styles$2.validationStatusIcon,
    name: validationStatusIconProps.name,
    color: validationStatusIconProps.color,
    spin: validationStatusIconProps.spin,
    type: "solid",
    size: 12
  }, (metadata === null || metadata === void 0 ? void 0 : metadata.testId) && {
    metadata: {
      testId: "".concat(metadata === null || metadata === void 0 ? void 0 : metadata.testId, "-icon")
    }
  })), /*#__PURE__*/React.createElement(Body, null, message));
};
var Validations = function (_a) {
  var id = _a.id,
    className = _a.className,
    validations = _a.validations;
  return /*#__PURE__*/React.createElement("div", {
    id: id,
    className: classnames(styles$3.validations, className)
  }, validations && validations.length > 0 && /*#__PURE__*/React.createElement(React.Fragment, null, validations.map(function (validation) {
    return /*#__PURE__*/React.createElement(ValidationMessage, {
      key: validation.key,
      message: validation.message,
      status: validation.status,
      metadata: validation.metadata
    });
  })));
};
var HorizontalFormField = function (_a) {
  var className = _a.className,
    children = _a.children,
    label = _a.label,
    htmlFor = _a.htmlFor,
    labelTooltipContent = _a.labelTooltipContent,
    labelTooltipMouseOutDelay = _a.labelTooltipMouseOutDelay,
    helperText = _a.helperText,
    hintText = _a.hintText,
    metadata = _a.metadata,
    validations = _a.validations,
    validationsId = _a.validationsId;
  var tooltipText = (helperText || labelTooltipContent) && /*#__PURE__*/React.createElement(React.Fragment, null, helperText && /*#__PURE__*/React.createElement("div", {
    className: styles$5.tooltipContent
  }, helperText), helperText && labelTooltipContent && /*#__PURE__*/React.createElement("br", null), labelTooltipContent && /*#__PURE__*/React.createElement("div", {
    className: styles$5.tooltipContent
  }, labelTooltipContent));
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles$5.horizontalFormField, className)
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", {
    className: styles$5.head
  }, /*#__PURE__*/React.createElement(Label, __assign({
    label: label,
    tooltipContent: tooltipText,
    tooltipMouseOutDelay: labelTooltipMouseOutDelay
  }, htmlFor ? {
    htmlFor: htmlFor
  } : {})), hintText && /*#__PURE__*/React.createElement(Body, {
    variant: "small",
    color: "text-default"
  }, hintText)), /*#__PURE__*/React.createElement("div", {
    className: styles$5.field
  }, children), /*#__PURE__*/React.createElement(Validations, {
    id: validationsId,
    validations: validations,
    className: styles$5.validations
  }));
};
var styles$1 = {
  "formField": "FormField-module__j-Q948X2__v7-3-4"
};
var VerticalFormField = function (_a) {
  var className = _a.className,
    children = _a.children,
    label = _a.label,
    htmlFor = _a.htmlFor,
    labelTooltipContent = _a.labelTooltipContent,
    labelTooltipMouseOutDelay = _a.labelTooltipMouseOutDelay,
    helperText = _a.helperText,
    hintText = _a.hintText,
    metadata = _a.metadata,
    validationsId = _a.validationsId,
    validations = _a.validations;
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles$1.formField, className)
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement(Label, __assign({
    label: label,
    tooltipContent: labelTooltipContent,
    tooltipMouseOutDelay: labelTooltipMouseOutDelay
  }, htmlFor ? {
    htmlFor: htmlFor
  } : {}, {
    hintText: hintText
  })), helperText && /*#__PURE__*/React.createElement(Body, {
    color: "text-subdued"
  }, helperText), /*#__PURE__*/React.createElement("div", null, children), /*#__PURE__*/React.createElement(Validations, {
    id: validationsId,
    validations: validations
  }));
};
var FormField = function (_a) {
  var horizontal = _a.horizontal,
    rest = __rest(_a, ["horizontal"]);
  return horizontal ? /*#__PURE__*/React.createElement(HorizontalFormField, __assign({}, rest)) : /*#__PURE__*/React.createElement(VerticalFormField, __assign({}, rest));
};
var styles = {
  "label": "FormFieldView-module__VlMHsYVA__v7-3-4",
  "horizontalFormFieldView": "FormFieldView-module__KMj6MuZy__v7-3-4"
};
var FormFieldView = function (_a) {
  var _b;
  var _c = _a.children,
    children = _c === void 0 ? null : _c,
    _d = _a.className,
    className = _d === void 0 ? '' : _d,
    _e = _a.horizontal,
    horizontal = _e === void 0 ? false : _e,
    _f = _a.metadata,
    metadata = _f === void 0 ? {} : _f,
    _g = _a.placeholder,
    placeholder = _g === void 0 ? '-' : _g,
    label = _a.label,
    _h = _a.labelMetadata,
    labelMetadata = _h === void 0 ? {} : _h;
  var hasChildren = Boolean(children);
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(className, (_b = {}, _b[styles.horizontalFormFieldView] = horizontal, _b))
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", {
    className: styles.label
  }, /*#__PURE__*/React.createElement(Heading, {
    variant: "small",
    as: "span",
    color: "text-default",
    metadata: labelMetadata
  }, label)), /*#__PURE__*/React.createElement(Body, {
    color: hasChildren ? 'text-default' : 'text-hint-text'
  }, hasChildren ? children : placeholder));
};
var validationStatusOutlines = {
  validating: 'default',
  info: 'default',
  valid: 'default',
  warning: 'warning',
  invalid: 'error'
};
var validationToSeverityMap = {
  validating: 0,
  info: 0,
  valid: 1,
  warning: 2,
  invalid: 3
};
var validationStatusToOutline = function (statuses) {
  if (!statuses || statuses.length === 0) return 'default';
  var highestSeverityStatus = Array.from(new Set(statuses)).sort(function (a, b) {
    return validationToSeverityMap[b] - validationToSeverityMap[a];
  })[0];
  return validationStatusOutlines[highestSeverityStatus];
};
var InputFormFieldWithRef = function (_a, ref) {
  var inputProps = _a.inputProps,
    id = _a.id,
    formFieldProps = __rest(_a, ["inputProps", "id"]);
  var generatedId = useId('inputFormField-');
  var inputId = id || generatedId;
  var validationsId = "validations-".concat(inputId);
  return /*#__PURE__*/React.createElement(FormField, __assign({}, formFieldProps, {
    htmlFor: inputId,
    validationsId: validationsId
  }), /*#__PURE__*/React.createElement(Input, __assign({}, inputProps, {
    id: inputId,
    "aria-describedby": validationsId,
    outline: formFieldProps.validations && validationStatusToOutline(formFieldProps.validations.map(function (x) {
      return x.status;
    })),
    ref: ref
  }, parseMetadata(suffixMetadata(formFieldProps.metadata, 'input')))));
};
var InputFormField = /*#__PURE__*/React.forwardRef(InputFormFieldWithRef);
var SelectFormField = function (_a) {
  var selectProps = _a.selectProps,
    id = _a.id,
    formFieldProps = __rest(_a, ["selectProps", "id"]);
  var generatedId = useId('selectFormField-');
  var selectId = id || generatedId;
  var validationsId = "validations-".concat(selectId);
  return /*#__PURE__*/React.createElement(FormField, __assign({}, formFieldProps, {
    htmlFor: selectId,
    validationsId: validationsId
  }), /*#__PURE__*/React.createElement(Select, __assign({}, selectProps, {
    triggerId: selectId,
    variant: "full-width",
    "aria-describedby": validationsId,
    outline: formFieldProps.validations && validationStatusToOutline(formFieldProps.validations.map(function (x) {
      return x.status;
    }))
  }, parseMetadata(suffixMetadata(formFieldProps.metadata, 'select')))));
};
var TextAreaFormFieldWithRef = function (_a, ref) {
  var textAreaProps = _a.textAreaProps,
    id = _a.id,
    formFieldProps = __rest(_a, ["textAreaProps", "id"]);
  var generatedId = useId('textFormField-');
  var textAreaId = id || generatedId;
  var validationsId = "validations-".concat(textAreaId);
  return /*#__PURE__*/React.createElement(FormField, __assign({}, formFieldProps, {
    htmlFor: textAreaId,
    validationsId: validationsId
  }), /*#__PURE__*/React.createElement(TextArea, __assign({}, textAreaProps, {
    id: textAreaId,
    "aria-describedby": validationsId,
    outline: formFieldProps.validations && validationStatusToOutline(formFieldProps.validations.map(function (x) {
      return x.status;
    })),
    ref: ref
  }, parseMetadata(suffixMetadata(formFieldProps.metadata, 'textarea')))));
};
var TextAreaFormField = /*#__PURE__*/React.forwardRef(TextAreaFormFieldWithRef);
export { FormField, FormFieldView, InputFormField, SelectFormField, TextAreaFormField };
//# sourceMappingURL=index.js.map
